/**********************************************************************
 program do obsugi czujnika siy "Kraft-Sensor", by Easy Soft 02/2003
 wersja 1.0
 **********************************************************************
 poczenia:
 - wywietlacz: dane P2, enable P0.6, r/s P0.7
 - klawiatura: wiersze P0.0,P0.1,P0.2; kolumny: P0.4, P0.5
 **********************************************************************/


#pragma SMALL
#include <reg8252.h>
#include <lcd8b.h>

#define	xsc	12				//odcita i rzdna skali
#define	ysc	1

#define	UP		191			//definicje kodw klawiszy
#define	DOWN	223
#define	LEFT	239
#define	RIGHT	127
#define	ONE	251
#define	TWO	254
#define	THREE	253

#define spiinit 0b00011111

// napisy uywane w programie
char code *Version_1 = "POWER SENSOR";
char code *Version_2 = "ver 1.0, dd 02/2003";
char code *aLine_1 = "BETRIEB AUTOMATIK";
char code *aLine_2 = ".";
char code *mLine_1 = "Linia menu 1  ";
char code *mLine_2 = "Linia menu 2. ";
char code *mLine_3 = "Linia menu 3..";

// wzorce znakw definiowanych - tabela 1
char code CGRom[65] = {0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xFF,0xC0,		// I				0x00
							  0xC0,0xC0,0xC0,0xC0,0xC0,0xFF,0xFF,0xC0,		// II				0x01
							  0xC0,0xC0,0xC0,0xC0,0xFF,0xFF,0xFF,0xC0,		// III			0x02
				           0xC0,0xC0,0xC0,0xFF,0xFF,0xFF,0xFF,0xC0,		// IIII			0x03
				           0xC0,0xC0,0xFF,0xFF,0xFF,0xFF,0xFF,0xC0,		// IIIII			0x04
							  0xC0,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xC0,		// IIIIII		0x05
							  0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xC0,		// IIIIIII		0x06
							  0xFF,0xFF,0xF3,0xF9,0xFC,0xF9,0xF3,0xFF,		// w prawo		0x07
				           0x00};

char code CTable[] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
char code *ctptr = &CTable;

char data buffer[4];
char data *ptrb;
char data temp, status;
sbit CS = P1^2;
sbit Relay = P3^5;

//obsuga klawiatury
char KeyScan()
{
	char i, loop, temp;						//zmienne pomocnicze

	while (loop < 30)
	{
		P0 = 0x2F;								//pierwsza kolumna = 0, odczyt wierszy i zapamitanie
		temp = P0;								//ich w starszej powce bajtu temp
		temp <<= 4;
		P0 = 0x1F;								//druga kolumna = 0, odczyt wierszy i zapamitanie ich
		i = P0;									//w zmiennej i
		i &= 0x0F;
		temp ^= i;								//przepisanie i do 4 modszych bitw temp
		loop++;
		if (temp == 0xFF) break;
	}
	return(temp);
}


//skala liniowa
void Scale(unsigned int value)
{
	char i;
	unsigned int pos = value / 8192;

	GotoXY(xsc,ysc);
	for (i = 0; i < pos; i++) LcdWrite(i);
	LcdWrite(' ');
	WriteTextXY(xsc+7,ysc,"X");
}


//tryb pracy automatycznej
void Auto()
{
	WriteTextXY(0,0,aLine_1);
	WriteTextXY(0,1,aLine_2);
}


//wywietlenie pozycji menu w trybie "rczne"
void displayManualMenu(char pos)
{
	GotoXY(0,0); LcdWrite(' ');			//usunicie poprzedniego znacznika
	GotoXY(0,1); LcdWrite(' ');
	switch(pos)
	{
		case 1:									//wywietlenie napisw menu
		case 2:
			WriteTextXY(1,0,mLine_1);
			WriteTextXY(1,1,mLine_2);
			break;
		case 3:
			WriteTextXY(1,0,mLine_2);
			WriteTextXY(1,1,mLine_3);
			break;
	}
	
	switch(pos)									//wywietlenie nowego znacznika
	{
		case 1:
			GotoXY(0,0);
			break;
		case 2:
		case 3:
			GotoXY(0,1);
			break;
	}
	LcdWrite(0x07);
}


//podjcie akcji w zalenoci od wybranej pozycji menu
void actionManualMenu(char pos)
{
}


//tryb nastaw wprowadzanych rcznie lub "uczenia si" kontrolera
void Manual()
{
	char x = 1;
	
	displayManualMenu(x);
	while (1)
	{
		temp = KeyScan();						//odczyt klawiatury
		switch (temp)
		{
			case  DOWN:							//klawisz "w d"
				if (++x > 3) x = 1;
				displayManualMenu(x);
				break;
			case UP:								//klawisz "w gr"
				if (--x == 0) x = 3;
				displayManualMenu(x);
				break;
			case THREE:							//klawisz zatwierdzenie (3)
				actionManualMenu(x);
				break;
		}
		Delay(150);
	}
}


//sending a byte using hardware SPI interface
void SPI_Write(char byteword)
{
	SPCR = spiinit;							//set SPI mode (interrupt disable,SPI disable,SPI master,cpol=cpha=1,sck=master clock/128)
	SPCR |= 0x40;								//SPI enable
	temp = SPSR;
	temp = SPDR;								//wcol & spif flags clear
	CS = 0;										//enable AD7714
	SPDR = byteword;							//write a byte to the SPI data register
	while (!(SPSR & 0x80));					//wait for spif flag set
	MOSI = 1;
	CS = 1;										//disable AD7714
}


//reading a byte from hardware SPI interface
void SPI_Read(char reglength)
{
	ptrb = &buffer;							//ptrb is pointer to buffer in ram memory
	SPCR = spiinit;							//set SPI mode (interrupt disable,SPI disable,SPI master,cpol=cpha=1,sck=master clock/128)
	SPCR |= 0x40;								//SPI enable
	CS = 0;										//enable AD7714
	while (P3 & 0x04);						//DRDY test
	while (reglength--)
	{
		SPDR = 0;								//start SPI receive
		while (!(SPSR & 0x80));				//wait for SPIF (data sent)
		*ptrb++ = SPDR;						//store a byte from data register
	}
	MOSI = 1;
	CS = 1;										//disable AD7714
}


//binary to hex conversion
void BinToHex(char x)
{
	temp = x/16;
	LcdWrite(*(ctptr+temp));
	temp = x%16;
	LcdWrite(*(ctptr+temp));
}


//program gwny
void main()
{
	SPI_Read(3);
	
	P3 |= 0x04;
	CS = 1;										//ad7714 = off
	Relay = 0;									//relay = off
	LcdInitialize();
	DefineSpecialCharacters(CGRom);
	WriteTextXY(0,0,Version_1);			//logo and version
	WriteTextXY(0,1,Version_2);
	Delay(1000);
	LcdClrScr();
	GotoXY(0,0);

	SPI_Write(0x24);							// Comm reg
	SPI_Write(0xF1); 							// Filter reg high, 8 bits
	
	SPI_Write(0x34); 							// Comm reg
	SPI_Write(0x40); 							// Filter reg low, 8 bits

	SPI_Write(0x14); 							// Comm reg
	SPI_Write(0x20); 							// Mode reg, 8 bits
	while (P3 & 0x04);						//DRDY test
	
	while(1)
	{
		SPI_Write(0x5C);						//data register read
		SPI_Read(4);

		GotoXY(0,0);							//only for testing purpose!
		ptrb = &buffer;
		BinToHex(*ptrb++);
		BinToHex(*ptrb++);
//		BinToHex(*ptrb++);
		BinToHex(*ptrb);
		Delay(1000);
	};
}

