/******************************************
  "bezpiecznik" z tensometrem
  (C)Easy Soft 06/2005
 ******************************************/
#include <reg8252.h>											//rejestry procesora
#include <wdt8252.h>											//obsuga Watchdog'a
#include <const51.h>											//definicje rnych uytecznych staych	
#include <eeprom8252.h>										//obsuga pamici eeprom
#include <spi.h>												//biblioteka funkcji obsugi SPI
	
//tekst umieszczany w kodzie programu
code BYTE CPY1[] = {"(C) 06/2005 EASY SOFT POLAND"};
code BYTE CPY2[] = {"FORCE FUSE WITH TENSOMETER"};
//tablice kodw zacze dla linijek led
code BYTE ledot1[] = {0xFF, 0x7F, 0x3F, 0x1F, 0x0F, 0x07, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03};
code BYTE ledot2[] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x7F, 0x3F, 0x1F, 0x0F, 0x07, 0x03, 0x01, 0x00};

//zmienne pomocnicze dla funkcji wywietlania diod LED
BYTE ot1num, ot2num;											//stan linijek (odwzorowanie bitowe)
bit lowerled;													//czy wywietlana "dolna" czy "grna" cz linijki LED
bit ledbetrieb;												//stan logiczny "1" oznacza, e LED "automat" ma si wieci
bit ledstoerung;												//stan logiczny "1" oznacza, e LED "awaria" ma si wieci
sbit OT1 = P0^1;												//sterowanie zaczaniem dolnej powki linijki diod LED; stan logiczny "0" zawieca
sbit OT2 = P0^2;												//sterowanie zaczaniem grnej powki linijki   - / / -

//zmienna do odmierzania sekund
WORD sec1;														//licznik jednostek sekund
BYTE sec100;													//licznik 0,01 sekundy

//tryb pracy interfejsu SPI
#define spiinit 0b00111111

//makra sterujce prac AD7714
#define ADSEL			ADCS = 0
#define ADDESEL		ADCS = 1
#define ADRDY_WAIT	while(ADRDY)

//zmienne, w ktrych przechowywane s wartoci nastaw podczas pracy
ULONG fMINIMUM, fMAXIMUM;

//stae skali LED
#define	CZAS_KALIBRACJI	294						//w sekundach
#define	LICZBA_DIOD_SKALI	14
const char STALA_KALIBRACJI_DLA_LED = CZAS_KALIBRACJI / LICZBA_DIOD_SKALI;

//adresy, pod ktrymi zapamitane s dane kalibracji
#define	CALIBR_OK	0
#define	eeMINIMUM	0x0002
#define	eeMAXIMUM	0x000A
#define	eeSTATUS		0x0020

//dane odczytane z przetwornika, sowo 14-bitowe
data ULONG danead;

//linie sterujce prac AD7714 (wszystkie linie: aktywne stany "0")
sbit ADCS = P1^4;											//zaczenie AD7714
sbit ADRST = P1^3;										//sprztowy restart AD7714
sbit ADRDY = P3^2;										//zgoszenie gotowoci danych do odbioru

//klawiatura (wszystkie linie: aktywne stany "0")
sbit kOBEN = P0^5;										//"w gr"
sbit kUNTEN = P0^6;										//"w d"
sbit kBETRIEB = P0^7;									//"tryb automatyczny"
sbit kLERNEN = P3^3;										//"tryb nauki"
sbit kFEHLER = P3^4;										//"kasowanie bdu"

//sygnalizacja bdu (przekanik)
sbit errorrelay = P0^0;




//procedura obsugi przerwania od timer'a 1: wywietlenie stanu diod led
void LedSend(void) interrupt 3 using 2
{
	TH1 = 0xDB;												//odwieenie zawartoci licznika timera 1 (T1 = 0xDBFF)
	OT1 = OT2 = 1;											//wyczenie napiecia zasilania obu "powek"
	lowerled = ~lowerled;								//negowanie bitu,od ktrego stanu zaley, ktra powka bdzie wywietlona
	if (lowerled) 											//jeli LOWERLED=1, to powka zawierajca rwnie diody statusu
	{
																//zaczenie (status_led=1) lub wyczenie (status_led<>1) diod statusu
		if (ledbetrieb) ot1num &= 0xFE; else ot1num |= 0x01;
		if (ledstoerung) ot1num &= 0xFD; else ot1num |= 0x02;
		P2 = ot1num; 										//wyprowadzenie przez port P2 stanu sowa zaczajcego diody
		OT2 = 0;												//zaczenie zasilania odpowiedniej 2-giej powki linijki LED
	}
	else 
	{
		P2 = ot2num;										//jak wyej, ale dla 1-szej powki (LOWERLED=0)
		OT1 = 0;
	}
																//odmierzanie sekund (jednoci i setne @ 100Hz)
	if (++sec100 >= 100)
	{
		sec1++;
		sec100 = 0;
	}
}


//opnienie <time x 0,1 sekundy> dla kwarcu 11,0592 MHz
void Delay(WORD time)
{
	BYTE i;
	
	while (time-- >0)
	{
		TF0 = 0;
		for (i = 0; i < 2; i++)								//ptla wykonywana 2x, bo podstaw czasu jest 0,05 sekundy
		{
			TH0 = 0x4B;											//wpisanie wartoci do rejestrw Timera 1
			TL0 = 0xFF;
			TR0 = 1;												//uruchomienie Timera 1
			while (!TF0);										//oczekiwanie na ustawienie flagi przepenienia
			TF0 = 0;												//zerowanie znacznika przepenienia Timera 1
		}
		TR0 = 0;													//zatrzymanie Timera 1
	}
}


//funkcja zapisuje dane do AD7714Y
void WriteToADReg(BYTE x)
{
	BYTE temp;
	
	SPION;														//SPE = 1;
	temp = SPSR;												//odczyt rejestrw w celu zerowania flagi SPIF
	temp = SPDR;
	ADSEL;														//zaczenie AD7714 na czas transmisji
	SPDR = x;													//zapis bajtu do rejestru danych SPI
	SPIWAIT;														//oczekiwanie na zakoczenie transmisji
	ADDESEL;														//wyczenie ukadu AD7714
}


//AD7714Y sygnalizuje gotowo danych wysyajc DRDY=0 (na INT0,
//czyli P3^3) funkcja odbiera dane od przetwornika i umieszcza je w 
//zmiennej globalnej <danead>
void GetSample(void) 
{
	char * ptrb = (char *)(&danead);						//danead jest typu long std konieczna konwersja wskanika
	char b;														//odbierane s 3 bajty
	
	WriteToADReg(0x5C);										//odczyt rejestru danych
	ADRDY_WAIT;													//oczekiwanie na zgoszenie danych do odbioru
	ADSEL;														//zaczenie AD7714Y
	*ptrb++ = 0;												//odbierana jest zmienna 3-bajtowa, aby dopasowa si do typu LONG pierwszy bajt musi by 0
	for (b = 0; b < 3; b++)
	{
		SPDR = 0;												//uruchomienie odbioru danych z SPI
		SPIWAIT;													//oczekiwanie na odbir bajtu
		*ptrb++ = SPDR;										//zapis bajtu pod adres wskazywany przez PTRB
	}
	ADDESEL;														//wyczenie AD7714Y
}


//wywietla linijk z diod LED (0-wyczone, 1 do 16)
void LedLine(BYTE lednum)
{
	ot1num = ledot1[lednum];
	ot2num = ledot2[lednum];
}


//obsuga bdu (wyjcie po naciniciu klawisza ST.LOESCHEN)
void TrybBledu()
{
	ledbetrieb = OFF;											//wyczenie diody LED trybu automatycznego
	errorrelay = CLOSE;										//zaczenie przekanika sygnalizacji bdu
	kFEHLER = ledstoerung = ON;							//zaczenie diody oraz wystawienie "1" na bit klawisza
	while (kFEHLER);											//dopki klawisz nie zostanie wcinity
	ledstoerung = OFF;										//wyczenie diody LED
	Delay(10);													//odczekanie 1 sekund
	errorrelay = OPEN;										//zwolnienie stykw przekanika
	ledbetrieb = ON;
}


//oczekiwanie na wcinicie klawisza LERNEN + migotanie LED
void Wait_4_LERNEN()
{
	while (kLERNEN) 
	{
		ledbetrieb = ledstoerung = ON;
		Delay(1);
		ledbetrieb = ledstoerung = OFF;
		Delay(1);
	}
}


//obsuga trybu nastaw; nacinicie klawisza BETRIEB wprowadza urzdzenie w tryb
//programowania automatycznego; nacinicie klawisza ST.LOESCHEN wprowadza tryb
//programowania rcznego; tryb programowania automatycznego koczy si po 5 minutach
//w czasie ktrych okrelane jest minimum i maksimum napre
void fdel()
{
	char i;
	for (i = 0; i < 255; i++);
}

void fBETRIEB()
{
	ledbetrieb = ON;
	fdel();
	ledbetrieb = OFF;
	fdel();
}

void TrybNastaw()
{
	ULONG temp;
	float x;
	WORD cnt;
	
	fMAXIMUM = 0;												//inicjalizacja wartoci zmiennych
	fMINIMUM = 0x00FFFFFF;
	
	ledbetrieb = ledstoerung = ON;						//zawiecane s obie diody LED
																	//teraz wybr, czy programowanie w trybuie automatycznym (kBETRIEB),
																	//czy te w trybie rcznym (kFEHLER)
	while ((kFEHLER != kPRESSED) && (kBETRIEB != kPRESSED));
	Delay (1);													//opnienie 100 ms i ponowny odczyt stanu klawiszy (wyeliminowanie drga itp)

	if (kFEHLER == kPRESSED)								//<- tryb nastaw rcznych
	{
//1.1  OCZEKIWANIE NA WCINICIE KLAWISZA LERNEN + MIGOTANIE DIOD
		Wait_4_LERNEN();										//oczekiwanie na wciniecie klawisza LERNEN
//1.2  POMIAR I NASTAWY MAKSIMUM
		sec1 = sec100 = 0;
		WDT_START;
//1.2.1 POMIAR WARTOCI MIN I MAX PRZEZ 5 MINUT
		while (sec1 < CZAS_KALIBRACJI)
		{
			WDT_RESET;
			LedLine(sec1/STALA_KALIBRACJI_DLA_LED);
			WDT_RESET;
			GetSample();
			if (fMINIMUM > danead) fMINIMUM = danead;
			if (fMAXIMUM < danead) fMAXIMUM = danead;
			WDT_RESET;
			if (cnt++ > 500)									//migotanie diody LED sygnalizuje pobieranie prbek
			{
				ledbetrieb = ~ledbetrieb;
				cnt = 0;
			}
		}
		WDT_RESET;
		ledbetrieb = OFF;
		x = fMAXIMUM;
		x *= 1.2;
		fMAXIMUM = (long)(x);
		ee_SaveLong(fMAXIMUM, eeMAXIMUM);				//zapamitanie zmierzonych wartoci
		x = fMINIMUM;
		x -= x*0.2;
		fMINIMUM = (long)(x);
		ee_SaveLong(fMINIMUM, eeMINIMUM);
		WDT_RESET;
		ADDESEL;													//wyczenie przetwornika A/D
//1.3.1 ZMIERZONO MIN I MAX, TERAZ DOSTRAJANIE RCZNE MAX
		temp = fMAXIMUM;
		ledbetrieb = OFF;
		LedLine(14);
		WDT_STOP;
		do 
		{
			if ((kOBEN == kPRESSED) && (temp < 0x00FFFFFF))
			{
				fBETRIEB();
				temp++;
			}
			if ((kUNTEN == kPRESSED) && (temp > 0)) 
			{
				fBETRIEB();
				temp--;
			}
			if (temp < fMAXIMUM) ledstoerung = ON; else ledstoerung = OFF;
		} while (kLERNEN != kPRESSED);
		while (kLERNEN == kPRESSED) Delay(1);			//oczekiwanie na zwolnienie LERNEN

//1.3.1 TERAZ DOSTRAJANIE RCZNE MIN
		temp = fMINIMUM;
		LedLine(1);
		do 
		{
			if ((kOBEN == kPRESSED) && (temp < 0x00FFFFFF)) 
			{
				fBETRIEB();
				temp++;
			}
			if ((kUNTEN == kPRESSED) && (temp > 0)) 
			{
				fBETRIEB();
				temp--;
			}
			if (temp > fMINIMUM) ledstoerung = ON; else ledstoerung = OFF;
		} while (kLERNEN != kPRESSED);

		WDT_START;
		ee_SaveLong(fMAXIMUM, eeMAXIMUM);				//zapamitanie ustawionych wartoci
		ee_SaveLong(fMINIMUM, eeMINIMUM);
		ee_savebyte(CALIBR_OK, eeSTATUS);				//zapamitanie informacji o wykonanej kalibracji
		WDT_RESET;
	} else

	if (kBETRIEB == kPRESSED)								//<- tryb nastaw automatycznych
	{
		ledstoerung = OFF;									//wyczenie diody STOERUNG
//2.3   POMIAR MAKSIMUM I MINIMUM
//2.3.1 OCZEKIWANIE NA WCINICIE KLAWISZA LERNEN + MIGOTANIE DIOD
		Wait_4_LERNEN();
		ledbetrieb = ON; 										//zaczenie tylko diody BETRIEB, przygotowanie do odmierzania czasu
		sec1 = sec100 = 0;
		WDT_START;
//2.3.2 POMIAR WARTOCI MIN I MAX PRZEZ 5 MINUT
		while (sec1 < CZAS_KALIBRACJI)
		{
			WDT_RESET;
			LedLine(sec1/STALA_KALIBRACJI_DLA_LED);
			WDT_RESET;
			GetSample();
			if (fMINIMUM > danead) fMINIMUM = danead;
			if (fMAXIMUM < danead) fMAXIMUM = danead;
			WDT_RESET;
			if (cnt++ > 500)									//migotanie diody LED sygnalizuje pobieranie prbek
			{
				ledbetrieb = ~ledbetrieb;
				cnt = 0;
			}
		}
		WDT_RESET;
		ledbetrieb = OFF;
		x = fMAXIMUM;
		x *= 1.2;
		fMAXIMUM = (long)(x);
		ee_SaveLong(fMAXIMUM, eeMAXIMUM);				//zapamitanie zmierzonych wartoci +/- 20%
		x = fMINIMUM;
		x = x - x*0.2;
		fMINIMUM = (long)(x);
		ee_SaveLong(fMINIMUM, eeMINIMUM);
		ee_savebyte(CALIBR_OK, eeSTATUS);				//zapamitanie informacji o wykonanej kalibracji
		WDT_STOP;
	}
}


//obsuga trybu automatycznego
void TrybRoboczy()
{
	ULONG temp;
	char llen;
	
	WDT_START;													//uruchomienie Watchog'a
	ledbetrieb = ON;
	temp = fMAXIMUM / LICZBA_DIOD_SKALI;				//warto maksymalna przez liczb diod skali
	for(;;)
	{
		WDT_RESET;												//zerowanie licznika WDT
		GetSample();
		llen = (char)(danead/temp);
		if (llen > 14) llen = 14;
		LedLine(llen);
		if ((danead < fMINIMUM) || (danead > fMAXIMUM)) TrybBledu();
	}
	WDT_STOP;													//zatrzymanie WDT na czas reakcji uytkownika
}


//program gwny
void main()
{
	BYTE i;

	WDT_RESET;													//zerowanie licznika WDT, jeli restart nastpi na skutek zadziaania WDT
	WDT_1024;													//zatrzymanie WDT i nastawa czasu na 512 ms
	TMOD = 0x11;												//Timer 1 jako 16-bitowy
	TH1 = 0xDF;													//przerwanie wywoywane z czstotliw.okoo 75Hz
	ET1 = ON;													//zezwolenie na przerwanie od timer'a 1
	TR1 = ON;													//uruchomienie timer'a 1
	EA = ON;														//zaczenie przerwa

	ADRST = ACTIVE;											//zerowanie przetwornika AD7714 (hardware reset)
	Delay(1);
	ADRST = DISACT;
	
	ledstoerung = ledbetrieb = ON;
	LedLine(14);
	errorrelay = OPEN;
	Delay(10);

	SPI_Init(spiinit);										//inicjalizacja i zaczenie interfejsu SPI
	SPION;														//zaczenie SPI (SPE=1)
                                 						//inicjalizacja i ustalenie trybu pracy AD7714Y
	SPI_Send(0x24);											//Comm reg
	SPI_Send(0x4F); 											//Filter reg high, 8 bits
	SPI_Send(0x34); 											//Comm reg
	SPI_Send(0xA0); 											//Filter reg low, 8 bits
	SPI_Send(0x14); 											//Comm reg
	SPI_Send(0x3C); 											//Mode reg, 8 bits (Mode:001,Gain:111,+00)
	ADRDY_WAIT;

	ledstoerung = ledbetrieb = OFF;						//wyczenie wszystkich diod LED
	LedLine(OFF);
	
	fMINIMUM = ee_GetLong(eeMINIMUM);					//pobranie nastaw z pamici EEPROM
	fMAXIMUM = ee_GetLong(eeMAXIMUM);
	
	if (ee_getbyte(eeSTATUS) == CALIBR_OK)				//sprawdzenie, czy wykonano kalibracj tzn. czy zapamitane s wartoci
	{																//minimalna i maksymalna
		kLERNEN = 1;
		if (kLERNEN == kPRESSED)							//sprawdzenie, czy klawisz LERNEN zosta wcinity
		{
			i = 5;                     					//LERNEN bdzie sprawdzany przez 5 sekund
			while ((i > 0) && (kLERNEN == kPRESSED))
			{
				ledbetrieb = ledstoerung = ON;
				Delay(1);
				ledbetrieb = ledstoerung = OFF;
				Delay(9);										//w sumie opnienie ok.1 sekundy
				i--;
			}
			if (i == 0) TrybNastaw();
		}
	} else TrybNastaw();										//tryb nastaw wywoywany rwnie przy braku kalibracji

	TrybRoboczy();												//uruchomienie automatycznego trybu pracy
} 

